@@ -2,26 +2,12 @@ import argparse, operator, shlex, shutil
 from pathlib import Path
 from typing import Dict
 
-import ffmpeg, sqlite_utils
 from catt.api import CattDevice
-from rich.prompt import Confirm
 
-from xklb import db, paths, utils
-from xklb.player import (
-    delete_media,
-    get_ordinal_media,
-    listen_chromecast,
-    local_player,
-    mark_media_deleted,
-    mark_media_watched,
-    mv_to_keep_folder,
-    override_sort,
-    printer,
-    remove_media,
-    socket_play,
-    watch_chromecast,
-)
-from xklb.utils import DEFAULT_PLAY_QUEUE, SC, cmd, log
+from xklb import db, paths, player, utils
+from xklb.player import get_ordinal_media, mark_media_deleted, override_sort
+from xklb.subtitle import externalize_subtitle
+from xklb.utils import DEFAULT_MULTIPLE_PLAYBACK, DEFAULT_PLAY_QUEUE, SC, cmd, log
 
 DEFAULT_PLAYER_ARGS_SUB = ["--speed=1"]
 DEFAULT_PLAYER_ARGS_NO_SUB = ["--speed=1.46"]
@@ -207,23 +193,25 @@ def fs_actions_usage(action, default_db):
         Duration to play (in seconds) while changing the channel
         library {action} --interdimensional-cable 40
         library {action} -4dtv 40
+
+        Playback multiple files at once
+        library {action} --multiple-playback    # one per display; or two if only one display detected
+        library {action} --multiple-playback 4  # play four media at once, divide by available screens
+        library {action} -m 4 --screen-name eDP # play four media at once on specific screen
+        library {action} -m 4 --loop --crop     # play four cropped videos on a loop
 """
 
 
 def parse_args(action, default_db, default_chromecast=""):
-    parser = argparse.ArgumentParser(
-        prog="library " + action,
-        usage=fs_actions_usage(action, default_db),
-    )
+    parser = argparse.ArgumentParser(prog="library " + action, usage=fs_actions_usage(action, default_db))
 
     parser.add_argument(
         "database",
         nargs="?",
         default=default_db,
-        help="Database file. If not specified a generic name will be used: audio.db, video.db, fs.db, etc",
+        help=f"Database file. If not specified a generic name will be used: {default_db}",
     )
 
-    # TODO: maybe try https://dba.stackexchange.com/questions/43415/algorithm-for-finding-the-longest-prefix
     parser.add_argument("--play-in-order", "-O", action="count", default=0, help=argparse.SUPPRESS)
     parser.add_argument("--sort", "-u", nargs="+", help=argparse.SUPPRESS)
     parser.add_argument("--random", "-r", action="store_true", help=argparse.SUPPRESS)
@@ -236,6 +224,19 @@ def parse_args(action, default_db, default_chromecast=""):
     parser.add_argument("--chromecast", "--cast", "-c", action="store_true", help=argparse.SUPPRESS)
     parser.add_argument("--with-local", "-wl", action="store_true", help=argparse.SUPPRESS)
     parser.add_argument("--interdimensional-cable", "-4dtv", type=int, help=argparse.SUPPRESS)
+    parser.add_argument(
+        "--multiple-playback",
+        "-m",
+        default=False,
+        nargs="?",
+        const=DEFAULT_MULTIPLE_PLAYBACK,
+        type=int,
+        help=argparse.SUPPRESS,
+    )
+    parser.add_argument("--screen-name", help=argparse.SUPPRESS)
+    parser.add_argument("--loop", action="store_true", help=argparse.SUPPRESS)
+    parser.add_argument("--crop", "--zoom", "--stretch", "--fit", "--fill", action="store_true", help=argparse.SUPPRESS)
+
     parser.add_argument("--portrait", "-portrait", action="store_true", help=argparse.SUPPRESS)
 
     parser.add_argument("--prefix", default="", help=argparse.SUPPRESS)
@@ -380,84 +381,21 @@ def transcode(next_video):
     return next_video
 
 
-def post_act(args, media_file: str):
-    mark_media_watched(args, media_file)
-    if args.post_action == "keep":
-        return
-
-    if args.action in [SC.tubelisten, SC.tubewatch]:
-        if args.post_action == "remove":
-            remove_media(args, media_file)
-        elif args.post_action == "ask":
-            if not Confirm.ask("Keep?", default=False):
-                remove_media(args, media_file)  # only remove metadata
-        else:
-            raise Exception("Unrecognized post_action", args.post_action)
-
-    if args.action in [SC.listen, SC.watch]:
-        if args.post_action == "remove":
-            remove_media(args, media_file)
-
-        elif args.post_action == "delete":
-            delete_media(args, media_file)
-
-        elif args.post_action == "delete-if-audiobook":
-            if "audiobook" in media_file.lower():
-                delete_media(args, media_file)
-
-        elif args.post_action == "ask":
-            if not Confirm.ask("Keep?", default=False):
-                delete_media(args, media_file)
-
-        elif args.post_action == "askkeep":
-            if not Confirm.ask("Keep?", default=False):
-                delete_media(args, media_file)
-            else:
-                mv_to_keep_folder(args, media_file)
-
-        else:
-            raise Exception("Unrecognized post_action", args.post_action)
-
-
-def externalize_subtitle(media_file):
-    subs = ffmpeg.probe(media_file)["streams"]
-
-    subtitles_file = None
-    if len(subs) > 0:
-        db = sqlite_utils.Database(memory=True)
-        db["subs"].insert_all(subs, pk="index")  # type: ignore
-        subtitle_index = db.execute_returning_dicts(
-            """select "index" from subs
-                order by
-                    lower(tags) like "%eng%" desc
-                    , lower(tags) like "%dialog%" desc
-                limit 1"""
-        )[0]["index"]
-        log.debug(f"Using subtitle {subtitle_index}")
-
-        subtitles_file = cmd("mktemp --suffix=.vtt --dry-run").stdout.strip()
-        cmd(
-            f'ffmpeg -nostdin -loglevel warning -txt_format text -i {media_file} -map "0:{subtitle_index}" "{subtitles_file}"'
-        )
-
-    return subtitles_file
-
-
 def chromecast_play(args, m):
     if args.action in [SC.watch, SC.tubewatch]:
-        catt_log = watch_chromecast(args, m, subtitles_file=externalize_subtitle(m["path"]))
+        catt_log = player.watch_chromecast(args, m, subtitles_file=externalize_subtitle(m["path"]))
     elif args.action in [SC.listen, SC.tubelisten]:
-        catt_log = listen_chromecast(args, m)
+        catt_log = player.listen_chromecast(args, m)
     else:
         raise NotImplementedError
 
     if catt_log:
-        if "Heartbeat timeout, resetting connection" in catt_log.stderr:
+        if catt_log.stderr is None or catt_log.stderr == "":
+            if not args.with_local:
+                raise Exception("catt does not exit nonzero? but something might have gone wrong")
+        elif "Heartbeat timeout, resetting connection" in catt_log.stderr:
             raise Exception("Media is possibly partially unwatched")
 
-        if catt_log.stderr == "":
-            raise Exception("catt does not exit nonzero? but something might have gone wrong")
-
 
 def is_play_in_order_lvl2(args, m, media_file):
     return any(
@@ -479,6 +417,7 @@ def play(args, m: Dict):
         if not media_path.exists():
             if args.is_mounted:
                 mark_media_deleted(args, media_file)
+            log.info("[%s]: Does not exist. Skipping...", media_file)
             return
         media_file = str(media_path)
 
@@ -490,17 +429,17 @@ def play(args, m: Dict):
     else:
         print(media_file)
 
+    args.player = player.parse(args, m, media_file)
+
     if args.chromecast:
         chromecast_play(args, m)
-
     elif args.interdimensional_cable:
-        socket_play(args, m)
-
+        player.socket_play(args, m)
     else:
-        local_player(args, m, media_file)
+        player.local_player(args, m, media_file)
 
     if args.action in [SC.listen, SC.watch, SC.tubelisten, SC.tubewatch] and not args.interdimensional_cable:
-        post_act(args, media_file)
+        player.post_act(args, media_file)
 
 
 audio_include_string = (
@@ -605,7 +544,7 @@ def construct_fs_query(args):
             if args.include:
                 args.sort = "random(), " + args.sort
             else:
-                limit = DEFAULT_PLAY_QUEUE * 2
+                limit = DEFAULT_PLAY_QUEUE * 16
         cf.append(f"and rowid in (select rowid from media order by random() limit {limit})")
 
     args.sql_filter = " ".join(cf)
@@ -645,7 +584,7 @@ def process_playqueue(args, construct_query=construct_fs_query):
     query, bindings = construct_query(args)
 
     if args.print:
-        return printer(args, query, bindings)
+        return player.printer(args, query, bindings)
 
     media = list(args.db.query(query, bindings))
 
@@ -658,15 +597,18 @@ def process_playqueue(args, construct_query=construct_fs_query):
 
     args.is_mounted = paths.is_mounted(list(map(operator.itemgetter("path"), media)), args.shallow_organize)
 
-    try:
-        for m in media:
-            play(args, m)
-    finally:
-        if args.interdimensional_cable:
-            args.sock.send((f"raw quit \n").encode("utf-8"))
-        Path(args.mpv_socket).unlink(missing_ok=True)
-        if args.chromecast:
-            Path(paths.CAST_NOW_PLAYING).unlink(missing_ok=True)
+    if args.multiple_playback:
+        player.multiple_player(args, media)
+    else:
+        try:
+            for m in media:
+                play(args, m)
+        finally:
+            if args.interdimensional_cable:
+                args.sock.send((f"raw quit \n").encode("utf-8"))
+            Path(args.mpv_socket).unlink(missing_ok=True)
+            if args.chromecast:
+                Path(paths.CAST_NOW_PLAYING).unlink(missing_ok=True)
 
 
 def watch():
