@@ -18,7 +18,10 @@ def parse_args(action):
     parser.add_argument("--verbose", "-v", action="count", default=0)
     args = parser.parse_args()
     if os.path.exists(args.mpv_socket):
-        args.mpv = MPV(start_mpv=False, ipc_socket=args.mpv_socket)
+        try:
+            args.mpv = MPV(start_mpv=False, ipc_socket=args.mpv_socket)
+        except ConnectionRefusedError:
+            Path(args.mpv_socket).unlink(missing_ok=True)
 
     log.info(utils.dict_filter_bool(args.__dict__))
 
@@ -62,6 +65,13 @@ def catt_stop(args):
     cmd("catt", *catt_device, "stop")
 
 
+def catt_pause(args):
+    catt_device = []
+    if args.chromecast_device:
+        catt_device = ["-d", args.chromecast_device]
+    cmd("catt", *catt_device, "play_toggle")
+
+
 def kill_process(name):
     if any([p in platform.system() for p in ["Windows", "_NT-", "MSYS"]]):
         cmd("taskkill", "/f", "/im", name)
@@ -75,10 +85,8 @@ def playback_stop():
     playing = _now_playing(args)
     if playing["mpv"]:
         args.mpv.command("loadfile", "/dev/null")  # make mpv exit with code 3
-    # else:
-    #     [kill_process(s) for s in ["python.*xklb", "bin/lb", "bin/library", "mpv"]]
 
-    if playing["catt"]:
+    if playing["catt"] or not any(playing.values()):
         kill_process("catt")
         catt_stop(args)
 
@@ -86,19 +94,32 @@ def playback_stop():
     Path(args.mpv_socket).unlink(missing_ok=True)
 
 
+def playback_pause():
+    args = parse_args("next")
+    playing = _now_playing(args)
+
+    if playing["catt"]:
+        catt_pause(args)
+
+    if playing["mpv"]:
+        args.mpv.command("cycle", "pause")
+        args.mpv.terminate()
+
+
 def playback_next():
     args = parse_args("next")
 
     playing = _now_playing(args)
 
     # TODO: figure out if catt or mpv is stale
-    if playing["catt"]:
+    # [kill_process(s) for s in ["python.*xklb", "bin/lb", "bin/library", "mpv"]]
+    if playing["catt"] or not any(playing.values()):
         Path(paths.CAST_NOW_PLAYING).unlink(missing_ok=True)
         catt_stop(args)
         if args.delete:
-            Path(playing["catt"]).unlink(missing_ok=True)
+            utils.trash(playing["catt"])
 
     if playing["mpv"]:
         args.mpv.command("playlist_next", "force")
         if args.delete:
-            Path(playing["mpv"]).unlink(missing_ok=True)
+            utils.trash(playing["mpv"])
