@@ -4,15 +4,125 @@ from io import StringIO
 from pathlib import Path
 from platform import system
 from random import randrange
-from shlex import quote
+from shlex import join, quote
 from shutil import which
 from time import sleep
-from typing import Union
+from typing import List, Union
 
+import ffmpeg, sqlite_utils
+from rich.prompt import Confirm
+from screeninfo import get_monitors
 from tabulate import tabulate
 
 from xklb import paths, utils
-from xklb.utils import SC, SQLITE_PARAM_LIMIT, cmd, cmd_interactive, human_time, log, os_bg_kwargs
+from xklb.utils import (
+    DEFAULT_MULTIPLE_PLAYBACK,
+    SC,
+    SQLITE_PARAM_LIMIT,
+    cmd,
+    cmd_interactive,
+    human_time,
+    log,
+    os_bg_kwargs,
+)
+
+
+def generic_player(args):
+    if platform.system() == "Linux":
+        player = ["xdg-open"]
+    elif any([p in platform.system() for p in ["Windows", "_NT-", "MSYS"]]):
+        if shutil.which("cygstart"):
+            player = ["cygstart"]
+        else:
+            player = ["start", ""]
+    else:
+        player = ["open"]
+    args.player_need_sleep = True
+    return player
+
+
+def calculate_duration(args, m):
+    start = 0
+    end = m["duration"]
+
+    if args.start:
+        if args.start == "wadsworth":
+            start = m["duration"] * 0.3
+        else:
+            start = args.start
+    if args.end:
+        if args.end == "dawsworth":
+            end = m["duration"] * 0.65
+        elif "+" in args.end:
+            end = (m["duration"] * 0.3) + args.end
+        else:
+            end = args.end
+
+    return start, end
+
+
+def find_xdg_application(media_file):
+    mimetype = cmd("xdg-mime", "query", "filetype", media_file).stdout
+    default_application = cmd("xdg-mime", "query", "default", mimetype).stdout
+    player_path = which(default_application.replace(".desktop", ""))
+    return player_path
+
+
+def parse(args, m=None, media_file=None) -> List[str]:
+    player = generic_player(args)
+    mpv = which("mpv.com") or which("mpv")
+
+    if args.player:
+        player = args.player
+        args.player_need_sleep = False
+
+    elif args.action in [SC.read] and media_file:
+        player_path = find_xdg_application(media_file)
+        log.info(player_path)
+
+    elif mpv:
+        args.player_need_sleep = False
+        player = [mpv]
+        if args.action in [SC.listen, SC.tubelisten]:
+            player.extend([f"--input-ipc-server={args.mpv_socket}", "--no-video", "--keep-open=no", "--really-quiet"])
+        elif args.action in [SC.watch, SC.tubewatch]:
+            player.extend(["--force-window=yes", "--really-quiet"])
+
+        if args.action in [SC.tubelisten, SC.tubewatch]:
+            player.extend(["--script-opts=ytdl_hook-try_ytdl_first=yes"])
+
+        if not args.multiple_playback:
+            player.extend(["--fs"])
+
+        if args.loop:
+            player.extend(["--loop-file=inf"])
+
+        if args.crop:
+            player.extend(["--panscan=1.0"])
+
+        if m and args.action in [SC.watch, SC.listen]:
+            start, end = calculate_duration(args, m)
+            if end != 0:
+                if start != 0:
+                    player.extend([f"--start={int(start)}", "--no-save-position-on-quit"])
+                if end != m["duration"]:
+                    player.extend([f"--end={int(end)}"])
+
+        if m and args.action == SC.watch:
+            if m["subtitle_count"] > 0:
+                player.extend(args.player_args_sub)
+            elif m.get("time_partial_first") is not None or m["size"] > 500 * 1000000:  # 500 MB
+                pass
+            else:
+                player.extend(args.player_args_no_sub)
+
+    elif system() == "Linux":
+        player_path = find_xdg_application(media_file)
+        if player_path:
+            args.player_need_sleep = False
+            player = [player_path]
+
+    return player
 
 
 def mv_to_keep_folder(args, media_file: str):
@@ -32,6 +142,26 @@ def mv_to_keep_folder(args, media_file: str):
         args.db.conn.execute("UPDATE media set path = ? where path = ?", [new_path, media_file])
 
 
+def moved_media(args, moved_files: Union[str, list], base_from, base_to):
+    moved_files = utils.conform(moved_files)
+    modified_row_count = 0
+    if len(moved_files) > 0:
+        df_chunked = utils.chunks(moved_files, SQLITE_PARAM_LIMIT)
+        for l in df_chunked:
+            with args.db.conn:
+                cursor = args.db.conn.execute(
+                    f"""UPDATE media
+                    SET path=REPLACE(path, '{quote(base_from)}', '{quote(base_to)}')
+                    where path in ("""
+                    + ",".join(["?"] * len(l))
+                    + ")",
+                    (*l,),
+                )
+                modified_row_count += cursor.rowcount
+
+    return modified_row_count
+
+
 def mark_media_watched(args, files):
     files = utils.conform(files)
     modified_row_count = 0
@@ -74,55 +204,11 @@ def mark_media_deleted(args, paths):
     return modified_row_count
 
 
-def moved_media(args, moved_files: Union[str, list], base_from, base_to):
-    moved_files = utils.conform(moved_files)
-    modified_row_count = 0
-    if len(moved_files) > 0:
-        df_chunked = utils.chunks(moved_files, SQLITE_PARAM_LIMIT)
-        for l in df_chunked:
-            with args.db.conn:
-                cursor = args.db.conn.execute(
-                    f"""UPDATE media
-                    SET path=REPLACE(path, '{quote(base_from)}', '{quote(base_to)}')
-                    where path in ("""
-                    + ",".join(["?"] * len(l))
-                    + ")",
-                    (*l,),
-                )
-                modified_row_count += cursor.rowcount
-
-    return modified_row_count
-
-
-def remove_media(args, deleted_files: Union[str, list], quiet=False):
-    deleted_files = utils.conform(deleted_files)
-    modified_row_count = 0
-    if len(deleted_files) > 0:
-        if not quiet:
-            if len(deleted_files) == 1:
-                print(f"[{deleted_files[0]}] Removing orphaned metadata")
-            else:
-                print(f"Removing {len(deleted_files)} orphaned metadata")
-
-        df_chunked = utils.chunks(deleted_files, SQLITE_PARAM_LIMIT)
-        for l in df_chunked:
-            with args.db.conn:
-                cursor = args.db.conn.execute(
-                    "delete from media where path in (" + ",".join(["?"] * len(l)) + ")",
-                    (*l,),
-                )
-                modified_row_count += cursor.rowcount
-
-    return modified_row_count
-
-
 def delete_media(args, media_file: str):
     if len(args.prefix) > 0:
         Path(media_file).unlink()
-    elif which("trash-put") is not None:
-        cmd("trash-put", media_file, strict=False)
     else:
-        Path(media_file).unlink()
+        utils.trash(media_file)
 
     mark_media_deleted(args, media_file)
 
@@ -137,7 +223,59 @@ def delete_playlists(args, playlists):
         )
 
 
+def post_act(args, media_file: str):
+    mark_media_watched(args, media_file)
+    if args.post_action == "keep":
+        return
+
+    if args.action in [SC.tubelisten, SC.tubewatch]:
+        if args.post_action == "remove":
+            mark_media_deleted(args, media_file)
+        elif args.post_action == "ask":
+            if not Confirm.ask("Keep?", default=False):
+                mark_media_deleted(args, media_file)
+        else:
+            raise Exception("Unrecognized post_action", args.post_action)
+
+    if args.action in [SC.listen, SC.watch]:
+        if args.post_action == "softdelete":
+            mark_media_deleted(args, media_file)
+
+        elif args.post_action == "delete":
+            delete_media(args, media_file)
+
+        elif args.post_action == "delete-if-audiobook":
+            if "audiobook" in media_file.lower():
+                delete_media(args, media_file)
+
+        elif args.post_action == "ask":
+            if not Confirm.ask("Keep?", default=False):
+                delete_media(args, media_file)
+
+        elif args.post_action == "askkeep":
+            if not Confirm.ask("Keep?", default=False):
+                delete_media(args, media_file)
+            else:
+                mv_to_keep_folder(args, media_file)
+
+        else:
+            raise Exception("Unrecognized post_action", args.post_action)
+
+
+def override_sort(string):
+    YEAR_MONTH = lambda var: f"cast(strftime('%Y%m', datetime({var}, 'unixepoch')) as int)"
+
+    return (
+        string.replace("month_created", YEAR_MONTH("time_created"))
+        .replace("month_modified", YEAR_MONTH("time_modified"))
+        .replace("random", "random()")
+        .replace("priority", " play_count, ntile(1000) over (order by size/duration) desc")
+    )
+
+
 def get_ordinal_media(args, path):
+    # TODO: maybe try https://dba.stackexchange.com/questions/43415/algorithm-for-finding-the-longest-prefix
+
     similar_videos = []
     candidate = str(path)
 
@@ -185,40 +323,6 @@ def get_ordinal_media(args, path):
     return similar_videos[0]
 
 
-def generic_player(args):
-    if platform.system() == "Linux":
-        player = ["xdg-open"]
-    elif any([p in platform.system() for p in ["Windows", "_NT-", "MSYS"]]):
-        if shutil.which("cygstart"):
-            player = ["cygstart"]
-        else:
-            player = ["start", ""]
-    else:
-        player = ["open"]
-    args.player_need_sleep = True
-    return player
-
-
-def calculate_duration(args, m):
-    start = 0
-    end = m["duration"]
-
-    if args.start:
-        if args.start == "wadsworth":
-            start = m["duration"] * 0.3
-        else:
-            start = args.start
-    if args.end:
-        if args.end == "dawsworth":
-            end = m["duration"] * 0.65
-        elif "+" in args.end:
-            end = (m["duration"] * 0.3) + args.end
-        else:
-            end = args.end
-
-    return start, end
-
-
 def watch_chromecast(args, m, subtitles_file=None):
     if "vlc" in args.player:
         catt_log = cmd(
@@ -246,7 +350,7 @@ def watch_chromecast(args, m, subtitles_file=None):
             catt_log = args.cc.play_url(m["path"], resolve=True, block=True)
 
     if subtitles_file:
-        Path(subtitles_file).unlink(missing_ok=True)
+        utils.trash(subtitles_file)
     return catt_log
 
 
@@ -303,68 +407,195 @@ def socket_play(args, m):
     sleep(args.interdimensional_cable)
 
 
-def find_xdg_application(media_file):
-    mimetype = cmd("xdg-mime", "query", "filetype", media_file).stdout
-    default_application = cmd("xdg-mime", "query", "default", mimetype).stdout
-    player_path = which(default_application.replace(".desktop", ""))
-    return player_path
+def is_odd(x):
+    if x & 1:
+        return True
+    else:
+        return False
+
+
+def geom(x_size, y_size, x, y):
+    return [f"--geometry={x_size}%x{y_size}%+{x}%+{y}%"]
+
+
+def geom_walk(v=1, h=1):
+    va = 100 // v
+    ha = 100 // h
+
+    geoms = []
+    for v_idx in range(v):
+        for h_idx in range(h):
+            x = (100 // max(1, v - 1)) * v_idx
+            y = (100 // max(1, h - 1)) * h_idx
+            geoms.append(geom(va, ha, x, y))
+
+    return geoms
+
+
+def vstack(display, qty):
+    mapper = {
+        1: [["--fs", f'--fs-screen-name="{display.name}"']],
+        2: geom_walk(v=2, h=1),
+        4: geom_walk(v=2, h=2),
+        6: geom_walk(v=2, h=3),
+        8: geom_walk(v=2, h=4),
+        9: geom_walk(v=3, h=3),
+        10: geom_walk(v=2, h=5),
+        12: geom_walk(v=3, h=4),
+        14: geom_walk(v=2, h=7),
+        15: geom_walk(v=3, h=5),
+        16: geom_walk(v=4, h=4),
+        18: geom_walk(v=3, h=6),
+        20: geom_walk(v=4, h=5),
+        21: geom_walk(v=3, h=7),
+        22: geom_walk(v=2, h=11),
+        24: geom_walk(v=4, h=6),
+        26: geom_walk(v=2, h=13),
+        28: geom_walk(v=4, h=7),
+        30: geom_walk(v=5, h=6),
+        32: geom_walk(v=4, h=8),
+        33: geom_walk(v=3, h=11),
+    }
+    if qty in mapper.keys():
+        holes = mapper[qty]
+    elif is_odd(qty):
+        holes = geom_walk(v=qty)
+    else:
+        v, h = divmod(qty, 2)
+        holes = geom_walk(v=v, h=v + h)
+
+    return [[f'--screen-name="{display.name}"', *hole] for hole in holes]
+
+
+def hstack(display, qty):
+    mapper = {
+        1: [["--fs", f'--fs-screen-name="{display.name}"']],
+        2: geom_walk(v=1, h=2),
+        4: geom_walk(v=2, h=2),
+        6: geom_walk(v=3, h=2),
+        8: geom_walk(v=4, h=2),
+        9: geom_walk(v=3, h=3),
+        10: geom_walk(v=5, h=2),
+        12: geom_walk(v=4, h=3),
+        14: geom_walk(v=7, h=2),
+        15: geom_walk(v=5, h=3),
+        16: geom_walk(v=4, h=4),
+        18: geom_walk(v=6, h=3),
+        20: geom_walk(v=5, h=4),
+        21: geom_walk(v=7, h=3),
+        22: geom_walk(v=11, h=2),
+        24: geom_walk(v=6, h=4),
+        26: geom_walk(v=13, h=2),
+        28: geom_walk(v=7, h=4),
+        30: geom_walk(v=6, h=5),
+        32: geom_walk(v=8, h=4),
+        33: geom_walk(v=11, h=3),
+    }
+    if qty in mapper.keys():
+        holes = mapper[qty]
+    elif is_odd(qty):
+        holes = geom_walk(h=qty)
+    else:
+        h, v = divmod(qty, 2)
+        holes = geom_walk(h=h, v=h + v)
+
+    return [[f'--screen-name="{display.name}"', *hole] for hole in holes]
+
+
+def get_display_by_name(displays, screen_name):
+    for d in displays:
+        if d.name == screen_name:
+            return [d]
+
+    display_names = '", "'.join([d.name for d in displays])
+    raise Exception(f'Display "{screen_name}" not found. I see: "{display_names}"')
+
+
+def get_multiple_player_template(args):
+    displays = get_monitors()
+    if args.screen_name:
+        displays = get_display_by_name(displays, args.screen_name)
+
+    if args.multiple_playback == DEFAULT_MULTIPLE_PLAYBACK and len(displays) == 1:
+        args.multiple_playback = 2
+    elif args.multiple_playback == DEFAULT_MULTIPLE_PLAYBACK and len(displays) > 1:
+        args.multiple_playback = len(displays)
+    elif args.multiple_playback < len(displays):
+        # play videos on supporting screens but not active one
+        displays = [d for d in displays if not d.is_primary]
+        displays = displays[: len(args.multiple_playback)]
+
+    min_media_per_screen, remainder = divmod(args.multiple_playback, len(displays))
+
+    displays.sort(key=lambda d: d.width * d.height)
+    players = []
+    for d_idx, display in enumerate(displays):
+        qty = min_media_per_screen
+        if remainder > 0 and (d_idx + 1) == len(displays):
+            qty += remainder
+
+        if display.width > display.height:  # wide
+            players.extend(vstack(display, qty))
+        else:  # tall or square: prefer horizontal split
+            players.extend(hstack(display, qty))
+
+    log.debug(players)
+
+    return players
+
+
+def multiple_player(args, media):
+    def open_player(template_args, m):
+        print(m["path"])
+        mp_args = ["--window-scale=1", "--no-border", "--no-keepaspect-window"]
+        return subprocess.Popen([*args.player, *mp_args, *template_args, "--", m["path"]], **os_bg_kwargs())
+
+    args.player = parse(args)
+
+    template = get_multiple_player_template(args)
+    players = []
+
+    try:
+        while media:
+            for t_idx, t in enumerate(template):
+                try:
+                    m = players[t_idx]
+                except IndexError:
+                    log.debug("%s IndexError", t_idx)
+                    m = media.pop()
+                    players.append({**m, "process": open_player(t, m)})
+                else:
+                    log.debug("%s Check if still running", t_idx)
+                    if m["process"].poll() is not None:
+                        r = utils.Pclose(m["process"])
+                        if r.returncode != 0:
+                            print("Player exited with code", r.returncode)
+                            log.debug(join(r.args))
+                            if not args.ignore_errors:
+                                exit(r.returncode)
+
+                        if args.action in [SC.listen, SC.watch, SC.tubelisten, SC.tubewatch]:
+                            post_act(args, m["path"])
+
+                        m = media.pop()
+                        players[t_idx] = {**m, "process": open_player(t, m)}
+
+            log.debug("-- A dragon slumbers over its hoard of %s media --", len(media))
+            sleep(0.5)  # stagger like Sir Michael Philip Jagger
+    finally:
+        for m in players:
+            m["process"].kill()
 
 
 def local_player(args, m, media_file):
-    player = generic_player(args)
-    mpv = which("mpv.com") or which("mpv")
-
-    if args.player:
-        player = args.player
-        args.player_need_sleep = False
-
-    elif args.action in [SC.read]:
-        player_path = find_xdg_application(media_file)
-        log.info(player_path)
-        pass
-
-    elif mpv:
-        args.player_need_sleep = False
-        player = [mpv]
-        if args.action in [SC.listen, SC.tubelisten]:
-            player.extend([f"--input-ipc-server={args.mpv_socket}", "--no-video", "--keep-open=no", "--really-quiet"])
-        elif args.action in [SC.watch, SC.tubewatch]:
-            player.extend(["--fs", "--force-window=yes", "--really-quiet"])
-
-        if args.action in [SC.tubelisten, SC.tubewatch]:
-            player.extend(["--script-opts=ytdl_hook-try_ytdl_first=yes"])
-
-        if args.action in [SC.watch, SC.listen]:
-            start, end = calculate_duration(args, m)
-            if end == 0:
-                return
-            if start != 0:
-                player.extend([f"--start={int(start)}", "--no-save-position-on-quit"])
-            if end != m["duration"]:
-                player.extend([f"--end={int(end)}"])
-
-        if args.action == SC.watch:
-            if m["subtitle_count"] > 0:
-                player.extend(args.player_args_sub)
-            elif m.get("time_partial_first") is not None or m["size"] > 500 * 1000000:  # 500 MB
-                pass
-            else:
-                player.extend(args.player_args_no_sub)
-
-    elif system() == "Linux":
-        player_path = find_xdg_application(media_file)
-        if player_path:
-            args.player_need_sleep = False
-            player = [player_path]
-
     if system() == "Windows" or args.action in [SC.watch, SC.tubewatch]:
-        r = cmd(*player, media_file, strict=False)
+        r = cmd(*args.player, media_file, strict=False)
     else:  # args.action in [SC.listen, SC.tubelisten]
-        r = cmd_interactive(*player, media_file, strict=False)
+        r = cmd_interactive(*args.player, media_file, strict=False)
     if r.returncode != 0:
         print("Player exited with code", r.returncode)
         if not args.ignore_errors:
-            exit(4)
+            exit(r.returncode)
 
     if args.player_need_sleep:
         if hasattr(m, "duration"):
@@ -400,7 +631,7 @@ def printer(args, query, bindings):
         exit(2)
 
     if "d" in args.print:
-        remove_media(args, list(map(operator.itemgetter("path"), db_resp)), quiet=True)
+        mark_media_deleted(args, list(map(operator.itemgetter("path"), db_resp)))
         if not "f" in args.print:
             return print(f"Removed {len(db_resp)} metadata records")
 
@@ -415,7 +646,7 @@ def printer(args, query, bindings):
             if not Path(f).exists():
                 mark_media_deleted(args, f)
                 return printer(args, query, bindings)
-            print(f)
+            print(quote(f))
         else:
             if not args.cols:
                 args.cols = ["path"]
@@ -463,14 +694,3 @@ def printer(args, query, bindings):
             duration = human_time(duration)
             if not "a" in args.print:
                 print("Total duration:", duration)
-
-
-def override_sort(string):
-    YEAR_MONTH = lambda var: f"cast(strftime('%Y%m', datetime({var}, 'unixepoch')) as int)"
-
-    return (
-        string.replace("month_created", YEAR_MONTH("time_created"))
-        .replace("month_modified", YEAR_MONTH("time_modified"))
-        .replace("random", "random()")
-        .replace("priority", " play_count, ntile(1000) over (order by size/duration) desc")
-    )
